package nl.amis.probe.webservice;

import java.util.GregorianCalendar;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

import javax.jws.soap.SOAPBinding;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import javax.xml.bind.annotation.XmlSeeAlso;
import javax.xml.ws.Action;

import nl.amis.probe.model.query.readProbeImpl;
import nl.amis.probe.model.query.readProbeRowImpl;
import nl.amis.probe.webservice.types.ObjectFactory;
import nl.amis.probe.webservice.types.TComponent;
import nl.amis.probe.webservice.types.TCheckProcessResponse;
import nl.amis.probe.webservice.types.TCheckProcessRequest;
import nl.amis.probe.webservice.types.TDeepPingResponse;
import nl.amis.probe.webservice.types.TDeepPingRequest;
import nl.amis.probe.webservice.types.TPingResponse;
import nl.amis.probe.webservice.types.TPingRequest;
import nl.amis.probe.webservice.types.TRoute;

import oracle.jbo.ApplicationModule;
import oracle.jbo.client.Configuration;

@XmlSeeAlso( { ObjectFactory.class })
@SOAPBinding(style = SOAPBinding.Style.DOCUMENT,
             parameterStyle = SOAPBinding.ParameterStyle.BARE)
@WebService(name = "ProbeServicePortType", serviceName = "ProbeService",
            targetNamespace = "nl.amis.services/probeservice/v1",
            portName = "ProbeServicePort",
            wsdlLocation = "/WEB-INF/wsdl/ProbeService.wsdl")
public class ProbeServicePortTypeImpl {

    private static final String APP_MODULE =
        "nl.amis.probe.model.ProbeAppModule";
    private static final String APP_MODULE_LOCAL = "ProbeAppModuleLocal";

    public ProbeServicePortTypeImpl() {
    }


    @SOAPBinding(parameterStyle = SOAPBinding.ParameterStyle.BARE)
    @Action(input = "nl.amis.services/probeservice/v1/ping",
            output = "nl.amis.services/probeservice/v1/ProbeServicePortType/pingResponse")
    @WebMethod(action = "nl.amis.services/probeservice/v1/ping")
    @WebResult(name = "PingResponse",
               targetNamespace = "nl.amis.messages/probeservice/v1",
               partName = "body")
    public TPingResponse ping(@WebParam(name = "PingRequest",
                                        partName = "body",
                                        targetNamespace = "nl.amis.messages/probeservice/v1")
        TPingRequest body) {

        TPingResponse result = new TPingResponse();
        try {
            result.setDateTimeRequest(body.getDateTimeRequest());
            result.setDateTimeResponse(getXMLGregorianCalendarNow());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        return result;
    }

    @SOAPBinding(parameterStyle = SOAPBinding.ParameterStyle.BARE)
    @Action(input = "nl.amis.services/probeservice/v1/deepPing",
            output = "nl.amis.services/probeservice/v1/ProbeServicePortType/deepPingResponse")
    @WebMethod(action = "nl.amis.services/probeservice/v1/deepPing")
    @WebResult(name = "DeepPingResponse",
               targetNamespace = "nl.amis.messages/probeservice/v1",
               partName = "body")
    public TDeepPingResponse deepPing(@WebParam(name = "DeepPingRequest",
                                                partName = "body",
                                                targetNamespace =
                                                "nl.amis.messages/probeservice/v1")
        TDeepPingRequest body) {

        ApplicationModule appModule = null;
        TDeepPingResponse result = new TDeepPingResponse();
        TRoute route = new TRoute();
        TComponent component = new TComponent();

        try {
            appModule =
                    Configuration.createRootApplicationModule(APP_MODULE, APP_MODULE_LOCAL);
            readProbeImpl readQuery =
                (readProbeImpl)appModule.findViewObject("readProbe");
            readQuery.executeQuery();
            readProbeRowImpl row = (readProbeRowImpl)readQuery.next();

            result.setDateTimeRequest(body.getDateTimeRequest());
            result.setDateTimeResponse(getXMLGregorianCalendarNow());
            component.setDateTimeResponse(getXMLGregorianCalendarNow());
            component.setName("ProbeServiceJAXWS");
            route.getComponent().add(component);
            result.setRoute(route);
        } catch (Exception e) {
            result.setDateTimeRequest(body.getDateTimeRequest());
            try {
                result.setDateTimeResponse(getXMLGregorianCalendarNow());
                component.setDateTimeResponse(getXMLGregorianCalendarNow());
            } catch (DatatypeConfigurationException f) {
                f.printStackTrace();
            }
            component.setErrorOccured(e.getMessage());
            route.getComponent().add(component);
            result.setRoute(route);
        } finally {
            if (appModule != null) {
                Configuration.releaseRootApplicationModule(appModule, true);
            }
        }

        return result;
    }

    @SOAPBinding(parameterStyle = SOAPBinding.ParameterStyle.BARE)
    @Action(input = "nl.amis.services/probeservice/v1/checkProcess",
            output = "nl.amis.services/probeservice/v1/ProbeServicePortType/checkProcessResponse")
    @WebMethod(action = "nl.amis.services/probeservice/v1/checkProcess")
    @WebResult(name = "CheckProcessResponse",
               targetNamespace = "nl.amis.messages/probeservice/v1",
               partName = "body")
    public TCheckProcessResponse checkProcess(@WebParam(name =
                                                        "CheckProcessRequest",
                                                        partName = "body",
                                                        targetNamespace =
                                                        "nl.amis.messages/probeservice/v1")
        TCheckProcessRequest body) {

        ApplicationModule appModule = null;
        TCheckProcessResponse result = new TCheckProcessResponse();
        try {
            appModule =
                    Configuration.createRootApplicationModule(APP_MODULE, APP_MODULE_LOCAL);
            readProbeImpl readQuery =
                (readProbeImpl)appModule.findViewObject("readProbe");
            readQuery.executeQuery();
            readProbeRowImpl rij = (readProbeRowImpl)readQuery.next();

            result.setProcessedResponse("ProbeServiceJAXWS: DB Sysdate=" +
                                        rij.getSysdate() + ", String=" +
                                        body.getProcessString() + ", Number=" +
                                        body.getProcessNumber().toString() +
                                        ", Date=" +
                                        body.getProcessDate().toString());
        } catch (Exception e) {
            e.printStackTrace();
            result.setProcessedResponse("ProbeServiceJAXWS: " +
                                        e.getMessage());
        } finally {
            if (appModule != null) {
                Configuration.releaseRootApplicationModule(appModule, true);
            }
        }
        return result;
    }

    private XMLGregorianCalendar getXMLGregorianCalendarNow() throws DatatypeConfigurationException {
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        DatatypeFactory datatypeFactory = DatatypeFactory.newInstance();
        XMLGregorianCalendar now =
            datatypeFactory.newXMLGregorianCalendar(gregorianCalendar);
        return now;
    }
}
